/***************************************************************************
                         FloatRenderCanvas.cpp
 ---------------------------------------------------------------------------
     begin                : 2003
     copyright            : (C) 2003 by Sabrina Skibak
     email                : sabrina.skibak@informatik.uni-ulm.de
 ***************************************************************************/

#include "FloatRenderCanvas.h"
#include "SDLRenderCanvas.h"
#include <SDL/SDL_image.h>
#include <png.h>

FloatRenderCanvas::FloatRenderCanvas(int f_XRes, int f_YRes):AbstractRenderCanvas(f_XRes, f_YRes)
{
	m_pData = NULL;
}

FloatRenderCanvas::~FloatRenderCanvas(void)
{
	delete [] m_pData;
	m_pData = NULL;
}

bool FloatRenderCanvas::Initialize()
{
	delete m_pData; m_pData = NULL;
	m_pData = new float[m_XResolution*m_YResolution*3];

	for (int i = 0; i < m_XResolution*m_YResolution*3; i++) m_pData[i] = 0;
	return true;
}

void FloatRenderCanvas::EnterRenderMode()
{
}

void FloatRenderCanvas::LeaveRenderMode()
{
}


void FloatRenderCanvas::DrawPixel(int f_X, int f_Y, float f_R, float f_G, float f_B)
{
	m_pData[(f_X+f_Y*m_XResolution)*3 + 0] = f_R;
	m_pData[(f_X+f_Y*m_XResolution)*3 + 1] = f_G;
	m_pData[(f_X+f_Y*m_XResolution)*3 + 2] = f_B;
}

void FloatRenderCanvas::DrawSafePixel(int f_X, int f_Y, float f_R, float f_G, float f_B)
{
	if ((f_X >= 0) && (f_X < m_XResolution) && (f_Y >= 0) && (f_Y < m_YResolution))
	{
		m_pData[(f_X+f_Y*m_XResolution)*3 + 0] = f_R;
		m_pData[(f_X+f_Y*m_XResolution)*3 + 1] = f_G;
		m_pData[(f_X+f_Y*m_XResolution)*3 + 2] = f_B;
	}
}

void FloatRenderCanvas::DrawPixelUchar(int f_X, int f_Y, unsigned char f_R, unsigned char  f_G, unsigned char  f_B)
{
	printf("DPU: not yet here.\n");
}

void FloatRenderCanvas::GetPixelColor(int f_X, int f_Y, float& f_R, float& f_G, float& f_B)
{
	f_R = m_pData[(f_X+f_Y*m_XResolution)*3 + 0];
	f_G = m_pData[(f_X+f_Y*m_XResolution)*3 + 1];
	f_B = m_pData[(f_X+f_Y*m_XResolution)*3 + 2];
}

float* FloatRenderCanvas::GetImageCopy(float* f_pDestination) 
{
	if (f_pDestination == NULL)
	{
		f_pDestination = new float[m_XResolution*m_YResolution*3];
	}
	memcpy(f_pDestination, m_pData, m_XResolution*m_YResolution*3 * sizeof(float));

	return f_pDestination;
}

void FloatRenderCanvas::Load(const char* f_pFilename)
{
	SDL_Surface* image;

	image = IMG_Load(f_pFilename);

	printf("Image File %s: %dx%d", f_pFilename, image->w, image->h);
	int p = image->format->BytesPerPixel;
	unsigned char rb, gb, bb;
	m_XResolution = image->w;
	m_YResolution = image->h;
	Initialize();

	for (int i = 0; i < m_XResolution* m_YResolution; i++)
	{
		SDL_GetRGB((Uint32)(*(Uint32*)((Uint8*) image->pixels + i*p)), image->format, &rb, &gb, &bb);
		m_pData[i*3+0] = rb /255.0f;
		m_pData[i*3+1] = gb /255.0f;
		m_pData[i*3+2] = bb /255.0f;
	}

	SDL_FreeSurface(image);

	printf(" loaded in FloatRenderCanvas.\n");
}

void FloatRenderCanvas::clearFloatBuffer(float f_Color)
 {
	 for (int x = 0;  x < m_XResolution; x++)
	 {
		 for (int y = 0; y < m_YResolution; y++)
		 {
			 m_pData[(x + y*m_XResolution)*3 + 0] = f_Color;
			 m_pData[(x + y*m_XResolution)*3 + 1] = f_Color;
			 m_pData[(x + y*m_XResolution)*3 + 2] = f_Color;
		 }
	 }
 }

void FloatRenderCanvas::clearFloatBuffer(float f_R, float f_G, float f_B)
{
	for (int x = 0;  x < m_XResolution; x++)
	 {
		 for (int y = 0; y < m_YResolution; y++)
		 {
			 m_pData[(x + y*m_XResolution)*3 + 0] = f_R;
			 m_pData[(x + y*m_XResolution)*3 + 1] = f_G;
			 m_pData[(x + y*m_XResolution)*3 + 2] = f_B;
		 }
	 }
}

void FloatRenderCanvas::CopyCanvas(SDLRenderCanvas* f_pCanvas)
{
	int xres, yres;
	if (m_XResolution > f_pCanvas->GetXResolution()) xres = f_pCanvas->GetXResolution();
	else xres = m_XResolution;
	if (m_YResolution > f_pCanvas->GetYResolution()) yres = f_pCanvas->GetYResolution();
	else yres = m_YResolution;

	for (int y = 0; y < yres; y++)
	{
		for (int x = 0; x < xres; x++)
		{
			f_pCanvas->DrawPixel(x, y, m_pData[(x + y*xres)*3 + 0],
				m_pData[(x + y*xres)*3 + 1],
				m_pData[(x + y*xres)*3 + 2]);
		}
	}
}

