/***************************************************************************
                         SDLOpenGLRenderCanvas.cpp
 ---------------------------------------------------------------------------
     begin                : Do Aug 14 2003
     copyright            : (C) 2002 by Holger Dammertz, Sabrina Skibak
     email                : holger.dammertz@informatik.uni-ulm.de
							sabrina.skibak@informatik.uni-ulm.de
 ***************************************************************************/

#include "SdlOpenGLRenderCanvas.h"

#include <stdlib.h>
#include <memory.h>

SDLOpenGLRenderCanvas::SDLOpenGLRenderCanvas(int f_XRes, int f_YRes, bool f_FullScreen)
                                            : SDLRenderCanvas(f_XRes, f_YRes, false, false)
{
	m_pScreen = NULL;
	m_pVideoInfo = NULL;
	m_fpQuitFunction = NULL;

	m_FullScreen = f_FullScreen;
}


SDLOpenGLRenderCanvas::~SDLOpenGLRenderCanvas(void)
{
}


void SDLOpenGLRenderCanvas::EnterRenderMode()
{
}

void SDLOpenGLRenderCanvas::LeaveRenderMode()
{
	glPointSize(1.0);
	glBegin(GL_POINTS);
	for (int y = 0; y < m_YResolution; y++)
		for (int x = 0; x < m_XResolution; x++)
		{
			glColor3fv(&m_pImageBuffer[(x+y*m_XResolution)*3]);
			glVertex2f(x, y);
		}
	glEnd();
	SDL_GL_SwapBuffers();
}

void SDLOpenGLRenderCanvas::SetQuitFunction(void (*f_fpQuitFunction)(int))
{
	m_fpQuitFunction = f_fpQuitFunction;
}


bool SDLOpenGLRenderCanvas::Initialize()
{
	if (m_pScreen != NULL)
	{
		fprintf(stderr, "\nSDLRenderCanvas::Initialize(): There is already a screen "
			"initialized.");
		return false;
	}
	if (SDL_Init(SDL_INIT_VIDEO) < 0)
	{
		fprintf(stderr, "\nSDLRenderCanvas::Initialize(): Video initialization failed: %s", 
			     SDL_GetError());
		return false;
	}

	SetVideoFlags(true);

	/* now we try to become a surface for rendering */
	m_pScreen = SDL_SetVideoMode(m_XResolution, m_YResolution, 32, m_VideoFlags);
	m_pImage = 	new Uint32[m_XResolution*m_YResolution];

	/* Verify that there is a surface */
	if (!m_pScreen)
	{
		fprintf(stderr, "\nSDLRenderCanvas::Initialize(): Video mode set failed: %s",
			SDL_GetError( ));
		return false;
	}

	printf("\nScreen set to %d x %d x %d",  m_pScreen->w,
	           m_pScreen->h, m_pScreen->format->BitsPerPixel);


	// initialize buffer
	m_pImageBuffer = new float[m_XResolution*m_YResolution*3];

    /* set viewing projection */
    glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluOrtho2D(0, m_XResolution, m_YResolution, 0);

    glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);
	glDisable(GL_ALPHA_TEST);


    return true;
}


void SDLOpenGLRenderCanvas::DrawAABB(float min_x, float min_y, float min_z, float max_x, float max_y, float max_z)
{
	glBegin(GL_LINE_LOOP);		
		glVertex3f(min_x, min_y, min_z);		
		glVertex3f(min_x, min_y, max_z);		
		glVertex3f(min_x, max_y, max_z);		
		glVertex3f(min_x, max_y, min_z);
	glEnd();
	glBegin(GL_LINE_LOOP);		
		glVertex3f(max_x, min_y, min_z);		
		glVertex3f(max_x, min_y, max_z);		
		glVertex3f(max_x, max_y, max_z);		
		glVertex3f(max_x, max_y, min_z);
	glEnd();
	glBegin(GL_LINES);
		glVertex3f(min_x, min_y, min_z);
		glVertex3f(max_x, min_y, min_z);
		glVertex3f(min_x, max_y, min_z);
		glVertex3f(max_x, max_y, min_z);
		glVertex3f(min_x, min_y, max_z);
		glVertex3f(max_x, min_y, max_z);
		glVertex3f(min_x, max_y, max_z);
		glVertex3f(max_x, max_y, max_z);
	glEnd();
}

void SDLOpenGLRenderCanvas::DrawPixel(int f_X, int f_Y, float f_R, float f_G, float f_B)
{
	m_pImageBuffer[(f_X+f_Y*m_XResolution)*3 + 0] = f_R;
	m_pImageBuffer[(f_X+f_Y*m_XResolution)*3 + 1] = f_G;
	m_pImageBuffer[(f_X+f_Y*m_XResolution)*3 + 2] = f_B;
}

void SDLOpenGLRenderCanvas::GetPixelColor(int f_X, int f_Y, float& f_R, float& f_G, float& f_B)
{
	f_R = m_pImageBuffer[(f_X+f_Y*m_XResolution)*3 + 0];
	f_G = m_pImageBuffer[(f_X+f_Y*m_XResolution)*3 + 1];
	f_B = m_pImageBuffer[(f_X+f_Y*m_XResolution)*3 + 2];
}

void SDLOpenGLRenderCanvas::DrawCircle(float f_Radius, float f_mx, float f_my, float f_Col_r,
									   float f_Col_g, float f_Col_b, float f_Opacity)
{
	/* draw a solid disc from a bunch of triangles */
	float angle = 0.0;
	float vectorY1 = f_my;
	float vectorX1 = f_mx;
	float vectorY = 0.0;
	float vectorX = 0.0;

	glBegin(GL_TRIANGLES);	
	glColor4f(f_Col_r, f_Col_g, f_Col_b, f_Opacity);
	for(int i = 0 ;i <= 360; i++)
	{
		angle = (float)(((double)i)/57.29577957795135);	
		vectorX = f_mx + (f_Radius*(float)sin((double)angle));
		vectorY = f_my + (f_Radius*(float)cos((double)angle));		
		glVertex2d(f_mx,f_my);
		glVertex2d(vectorX1,vectorY1);
		glVertex2d(vectorX,vectorY);
		vectorY1=vectorY;
		vectorX1=vectorX;	
	}
	glEnd();
}



/***************************************************************************
 *    Protected Methods
 ***************************************************************************/


void SDLOpenGLRenderCanvas::SetVideoFlags(bool f_PrintInfo)
{
	m_pVideoInfo = SDL_GetVideoInfo();
	if (!m_pVideoInfo)
	{
		fprintf(stderr, "\nSDLRenderCanvas::SetVideoFlags(bool f_PrintInfo): Could"
			" not get VideoInfo from SDL: %s", SDL_GetError());
		return;
	}

	m_VideoFlags = SDL_HWPALETTE;  // Store the palette in hardware
	m_VideoFlags |= SDL_OPENGL;
	m_VideoFlags |= SDL_GL_DOUBLEBUFFER;  // Enable double buffering
	// !!TODO: when using OpenGL we could also allow Resizing here...

	if (m_FullScreen) m_VideoFlags |= SDL_FULLSCREEN;       // Enable fullscreen
	if (m_pVideoInfo->blit_hw) m_VideoFlags |= SDL_HWACCEL; // This checks if hardware blits can be done 

	/* This checks to see if surfaces can be stored in memory */
	if (f_PrintInfo)
	{
		printf("\nScreen Information: ");
		//printf("\n  Resolution: %dx%d", m_XResolution, m_YResolution);
		if (m_pVideoInfo->hw_available)
		{
			m_VideoFlags |= SDL_HWSURFACE;
			printf("\n  Surface is hardware.");
		}
		else
		{
			m_VideoFlags |= SDL_SWSURFACE;
			printf("\n  Surface is software.");
		}
		if (m_pVideoInfo->blit_hw)
		{
			printf("\n  Hardware accelerated blit enabled.");
		}
		else
		{
			printf("\n  Hardware accelerated blit disabled.");
		}
	}
}