// Here you find sample code for using a render canvas, loading and saving
// images and using keyboard/mouse events

#include <windows.h>

#include <GL/GL.h>
#include <GL/glu.h>

#include "FloatRenderCanvas.h"
#include "SDLRenderCanvas.h"
#include "SDLOpenGLRenderCanvas.h"
#include "utils.h"
#include "rtMath.h"

int ResX, ResY;
SDLRenderCanvas* canvas;
FloatRenderCanvas* floatCanvas;
bool mousePressed = false;


class MainEventHandler : public SDLEventConsumer
{
	bool Consume(SDL_Event& m_rEvent)
	{
		switch(m_rEvent.type)
		{
		case SDL_KEYDOWN:
			{
				switch(m_rEvent.key.keysym.sym)
				{
				case SDLK_c:
					canvas->clearCanvas(1.0, 1.0, 1.0);
					return true;
				case SDLK_s:
					canvas->SaveAsPNG("test2.png");
					return true;
				default:
					return false;
				}
			}
		case SDL_MOUSEMOTION:
			if (mousePressed)
			{
				canvas->DrawPixel(m_rEvent.motion.x, m_rEvent.motion.y, 1.0, 0.0, 0.0);
			}
			/*printf("Mouse moved by %d,%d to (%d,%d)\n", 
				m_rEvent.motion.xrel, m_rEvent.motion.yrel,
				m_rEvent.motion.x, m_rEvent.motion.y);*/
			break;
		case SDL_MOUSEBUTTONDOWN:
			mousePressed = !mousePressed;
			/*printf("Mouse button %d pressed at (%d,%d)\n",
				m_rEvent.button.button, m_rEvent.button.x, m_rEvent.button.y);*/
			break;
			return false;
		default:
			return false;
		}
		return false;
	}
};



int main(int argc, char* argv[])
{
	ResX = 512;
	ResY = 512;
	canvas = new SDLRenderCanvas(ResX, ResY, false, true);
	canvas->Initialize();
	canvas->AddEventConsumer(new MainEventHandler());// the event handling in the main loop

	//When the FloatRenderCanvas is used, it has to be copied to the SDLRenderCanvas for display.
	floatCanvas = new FloatRenderCanvas(ResX, ResY);
	floatCanvas->Initialize();
	
	/* The load method adapts the FloatCanvas resolution to the image size!!
	  Loading images is just possible with the FloatRenderCanvas */
	floatCanvas->Load("Wasserlilien.jpg"); 
	floatCanvas->CopyCanvas(canvas);
	//floatCanvas->SaveAsPNG("test.png");

	while(true)
	{
		canvas->HandleEvents();
		canvas->EnterRenderMode();

		// !! INSERT YOUR RAY TRACER HERE !!

		for (int y = 0; y < ResX/4; y++)
		{
			canvas->HandleEvents();
			for (int x = 0; x < ResY/4; x++)
			{
				//Draw in the SDLRenderCanvas here
				canvas->DrawPixel(x, y, 0.67f, 0.32f, 0.84f); 
			}

		}
		canvas->LeaveRenderMode();
	}

	return 0;
}


