#include "utils.h"
#include "rgbe.h"
#include <stdio.h>
#include <malloc.h>
#include <math.h>

using namespace std;

//gibt RGB-Float-Picture (3*32Bit * xres*yres) zurck und belegt xres bzw. yres mit der Picture-Gre
float* LoadHDRI(const char *filename, int &xres, int &yres)
{
  //File ffnen und HDRI einlesen
  FILE *f = fopen(filename,"rb");

  if (f)
  {
	RGBE_ReadHeader(f,&xres,&yres,NULL);
	float* hdr_picture = (float *)malloc(sizeof(float)*3*xres*yres);
	RGBE_ReadPixels_RLE(f,hdr_picture,xres,yres);

	fclose(f);
    return(hdr_picture);
  }
  else
  {
        xres=-1;
        yres=-1;
        return(0);
  }  
}


//gibt RGB-Float-Picture (3*32Bit * xres*yres) zurck und belegt xres bzw. yres mit der Picture-Gre
void SaveHDRI(const char *filename, int xres, int yres, float* data)
{
	//File ffnen und HDRI einlesen
	FILE *f = fopen(filename,"wb");

	if (f == NULL)
	{
		fprintf(stderr, "error creating %s", filename);
		return;
	}

	RGBE_WriteHeader(f, xres, yres, 0);
	RGBE_WritePixels_RLE(f, data, xres, yres);
	fclose(f);
}



//gibt XYZ-Float-Array (face_point_coords; 3*32Bit * 3 * num_faces) und UV-Float-Array (face_tex_coords; 2*32Bit * 3 * num_faces) zurck und belegt num_faces
void LoadRAWTriangles(const char *filename, float* &face_point_coords, float* &face_tex_coords, long &num_faces)
{
  FILE *myfile; 
     
  myfile=fopen(filename,"rb");
  if (myfile)
  {
	  printf("Loading RAW Triangles from %s", filename);

      fseek (myfile,0,SEEK_END);
      unsigned long filesize = ftell(myfile);
      fseek (myfile,0,SEEK_SET);

      //alles in einem rutsch laden (3 vertices pro face // 1 pointcoord (XYZ) + 1 texcoord (UV) pro vertex)

      num_faces=filesize/((sizeof(float)*3+sizeof(float)*2)*3);

	  printf(" with %d Triangles", num_faces);
      
	  //face_point_coords=(float *)malloc(num_faces*3*sizeof(float)*3); 
	  face_point_coords = new float[num_faces*3*3]; 
      fread(face_point_coords,num_faces*3*sizeof(float)*3,1,myfile); 
      //face_tex_coords=(float *)malloc(num_faces*3*sizeof(float)*2); 
	  face_tex_coords = new float[num_faces*3*2]; 
      fread(face_tex_coords,num_faces*3*sizeof(float)*2,1,myfile);
      
      fclose(myfile);
	  printf("finished.\n");
  }
  else
  {
	  fprintf(stderr, "Could not load RAW Triangles from %s.", filename);
      num_faces=-1;
      face_point_coords=0;
      face_tex_coords=0;
  }
}

//gibt XYZ-Float-Array (3*32Bit * 4*4 * num_bezierPatches) zurck und belegt num_bezierPatches
float* LoadRAW4x4BezierPatches(const char *filename, long &num_bezierPatches)
{
  FILE   *myfile;
     
  myfile=fopen(filename,"rb");
  if (myfile)
  {
      fseek (myfile,0,SEEK_END);
      unsigned long filesize = ftell(myfile);
      fseek (myfile,0,SEEK_SET);

      //alles in einem rutsch laden (4x4 vertices (XYZ) pro patch)
      num_bezierPatches=filesize/(sizeof(float)*3*4*4);

      //float* bezierPatch=(float *)malloc(filesize);
	  float* bezierPatch = new float[filesize];
      fread(bezierPatch,filesize,1,myfile);

      fclose(myfile);
      return(bezierPatch);
  }
  else
  {
      num_bezierPatches=-1;
      return(0);
  }
}